#!/www/python/bin/python
"""$URL: svn+ssh://svn/repos/trunk/dulcinea/bin/sitecheck.py $
$Id: sitecheck.py 26135 2005-02-15 21:36:23Z dbinger $
"""

import os
import sys
from dulcinea.traversal import check_exports

if len(sys.argv) != 2:
    print "Usage: sitecheck.py site"
    raise SystemExit

site = sys.argv[1]
print
print site
os.environ['SITE'] = site
check_exports()
