#!/usr/bin/env python
"""
$URL: svn+ssh://svn/repos/trunk/dulcinea/bin/unknown.py $
$Id: unknown.py 26302 2005-03-08 14:06:55Z dbinger $
"""

from dulcinea.code_util import report_unknown_names, main

main(report_unknown_names)
